import sys
from distutils.version import StrictVersion
from dataclasses import dataclass


def my_close_open_files(verbose):
    """

    :param verbose:
    """
    open_files = tables.file._open_files

    are_open_files = len(open_files) > 0

    if verbose and are_open_files:
        sys.stderr.write("Closing remaining open files:")

    if StrictVersion(tables.__version__) >= StrictVersion("3.1.0"):
        # make a copy of the open_files.handlers container for the iteration
        handlers = list(open_files.handlers)
    else:
        # for older versions of pytables, setup the handlers list from the
        # keys
        keys = open_files.keys()
        handlers = []
        for key in keys:
            handlers.append(open_files[key])

    for fileh in handlers:
        if verbose:
            sys.stderr.write("%s..." % fileh.filename)

        fileh.close()

        if verbose:
            sys.stderr.write("done")

    if verbose and are_open_files:
        sys.stderr.write("\n")


def dataclass_with_default_init(_cls=None, *args, **kwargs):
    """

    :rtype: object
    """
    def wrap(cls):
        # Save the current __init__ and remove it so dataclass will
        # create the default __init__.
        user_init = getattr(cls, "__init__")
        delattr(cls, "__init__")

        # let dataclass process our class.
        result = dataclass(cls, *args, **kwargs)

        # Restore the user's __init__ save the default init to __default_init__.
        setattr(result, "__default_init__", result.__init__)
        setattr(result, "__init__", user_init)

        # Just in case that dataclass will return a new instance,
        # (currently, does not happen), restore cls's __init__.
        if result is not cls:
            setattr(cls, "__init__", user_init)

        return result

    # Support both dataclass_with_default_init() and dataclass_with_default_init
    if _cls is None:
        return wrap
    else:
        return wrap(_cls)




# -*- coding: utf-8 -*-

"""Main module of pylexique."""

from collections import OrderedDict
import pkg_resources
import json
from zipfile import ZipFile
import atexit
from dataclasses import dataclass
from typing import ClassVar

_RESOURCE_PACKAGE = __name__

PYLEXIQUE_DATABASE = '/'.join(('Lexique383', 'lexique383.xlsb'))
HOME_PATH = '/'.join(('Lexique', ''))

LEXIQUE383_FIELD_NAMES = ['ortho', 'phon', 'lemme', 'cgram', 'genre', 'nombre', 'freqlemfilms2', 'freqlemlivres',
                          'freqfilms2',
                          'freqlivres', 'infover', 'nbhomogr', 'nbhomoph', 'islem', 'nblettres', 'nbphons', 'cvcv',
                          'p_cvcv',
                          'voisorth', 'voisphon', 'puorth', 'puphon', 'syll', 'nbsyll', 'cv_cv', 'orthrenv', 'phonrenv',
                          'orthosyll', 'cgramortho', 'deflem', 'defobs', 'old20', 'pld20', 'morphoder', 'nbmorph']


LEXIQUE = OrderedDict()



class LexEntryTypes:
    """
    Hint type information.

    """
    ortho = str
    phon = str
    lemme = str
    cgram = str
    genre = str
    nombre = str
    freqlemfilms2 = float
    freqlemlivres = float
    freqfilms2 = float
    freqlivres = float
    infover = str
    nbhomogr = int
    nbhomoph = int
    islem = bool
    nblettres = int
    nbphons = int
    cvcv = str
    p_cvcv = str
    voisorth = int
    voisphon = int
    puorth = int
    puphon = int
    syll = str
    nbsyll = int
    cv_cv = str
    orthrenv = str
    phonrenv = str
    orthosyll = str
    cgramortho = str
    deflem = float
    defobs = int
    old20 = float
    pld20 = float
    morphoder = str
    nbmorph = int


class Lexique383:
    """
    This is the class handling the lexique database.
    It provides method for interacting with the Lexique DB
    and retrieve lexical items.
    All the lexical items are then stored in an Ordered Dict called

    :param lexique_path: string.
        Path to the lexique csv file.
    """

    file_name = pkg_resources.resource_filename(_RESOURCE_PACKAGE, PYLEXIQUE_DATABASE)

    def __init__(self, lexique_path=None):
        self.lexique_path = lexique_path
        self.lexique = OrderedDict()
        if lexique_path:
            print('parsing Lexique383')
            self.parse_lexique(self.lexique_path)
            # with ZipFile(pkg_resources.resource_stream(
            #         _RESOURCE_PACKAGE, PYLEXIQUE_DATABASE)) as content:
            #     with content.open('Lexique383.pickle', mode='w') as archive:
            #         joblib.dump(self.lexique , archive)
        else:
            # self.parse_lexique(self.lexique_path)
            # with ZipFile(pkg_resources.resource_stream(
            #         _RESOURCE_PACKAGE, PYLEXIQUE_DATABASE)) as content:
            #     with content.open('Lexique383.pickle', mode='r') as archive:
            #         joblib.load(archive)
            pass
        print('Lexique 383 loaded successfully')
        return

    def __repr__(self):
        return '{0}.{1}'.format(__name__, self.__class__.__name__)

    def __len__(self):
        return len(self.lexique)

    def parse_lexique(self, lexique_path):
        """
        | Parses the given lexique file and creates a hdf5 table to store the data.

        :param lexique_path: string.
            Path to the lexique csv file.
        :return: PyTables.Table
        """
        with open(lexique_path, 'r', encoding='utf-8', errors='ignore') as csv_file:
            content = csv_file.readlines()
            lexique383_db = (content[1:])
            self.create_db(lexique383_db)
        return

    def create_db(self, lexicon):
        """
        | Creates an hdf5 table populated with the entries in lexique if it does not exist yet.
        | It stores the hdf5 database for fast access.

        :param lexicon: Iterable.
            Iterable containing the lexique383 entries.
        :return: PyTables.Table
        """
        errors = {}
        for i, row in enumerate(lexicon):
            row_fields = row.strip().split('\t')
            formatted_row_fields = []
            for field, value in zip(LEXIQUE383_FIELD_NAMES, row_fields):
                if getattr(LexEntryTypes, field) == float:
                    value = value.replace(',', '.')
                try:
                    formatted_value = getattr(LexEntryTypes, field)(value)
                except ValueError:
                    errors[value] = field
                    formatted_value = value
                finally:
                    formatted_row_fields.append(formatted_value)
            if row_fields[0] in self.lexique and not isinstance(self.lexique[row_fields[0]], list):
                self.lexique[row_fields[0]]= [self.lexique[row_fields[0]]]
                self.lexique[row_fields[0]].append(formatted_row_fields)
            elif row_fields[0] in self.lexique and isinstance(self.lexique[row_fields[0]], list):
                self.lexique[row_fields[0]].append(LexItem(formatted_row_fields))
            else:
                self.lexique[row_fields[0]] = LexItem(formatted_row_fields)
        return


@dataclass(init=False, repr=True, eq=True, order=False, unsafe_hash=False, frozen=False)
class LexItem:
    """
    | This class defines the lexical items in Lexique383.
    | It uses slots for memory efficiency.

    :param row_fields:
    """
    _s: ClassVar[list] = LEXIQUE383_FIELD_NAMES + ['_name_']
    __slots__ = _s
    _name_: str
    for attr in LEXIQUE383_FIELD_NAMES:
        attr: LexEntryTypes

    def __init__(self, row_fields):
        fields = row_fields
        setattr(self, '_name_', fields[0])
        for attr, value in zip(LEXIQUE383_FIELD_NAMES, fields):
            setattr(self, attr, value)
        return


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-

"""Top-level package for pylexique."""

__author__ = """SekouDiaoNlp"""
__email__ = 'diao.sekou.nlp@gmail.com'
__version__ = '1.1.1'
__copyright__ = "Copyright (c) 2021, SekouDiaoNlp"
__credits__ = ("Lexique383",)
__license__ = "MIT"
__maintainer__ = "SekouDiaoNlp"
__status__ = "Production"

from collections import OrderedDict
import pkg_resources
import json
import pkg_resources
from .utils import my_close_open_files
from .pylexique import Lexique383

# PYLEXIQUE_DATABASE = '/'.join(('Lexique383', 'lexique383.h5'))
# HOME_PATH = '/'.join(('Lexique', ''))

_RESOURCE_PACKAGE = 'pylexique'
_RESOURCE_PATH = pkg_resources.resource_filename(_RESOURCE_PACKAGE, 'Lexique383/Lexique383.txt')

# LEXIQUE = Lexique383(_RESOURCE_PATH).lexique
# lexique383, LEXIQUE = Lexique383()  # Use only if the hdf5 file exists.
